package com.huiju.estateagents.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.CommonUtils;
import com.huiju.estateagents.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.huiju.estateagents.service.ITaShopTypeService;
import com.huiju.estateagents.entity.TaShopType;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
    * 商户分类  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-11-03
 */

@Api(tags = "商户分类 ")
@RestController
@RequestMapping("/api")
public class TaShopTypeController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaShopTypeController.class);

    @Autowired
    public ITaShopTypeService iTaShopTypeService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/shop-type",method= RequestMethod.GET)
    // @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taShopTypeList(@ApiParam("客户端") @PathVariable String client,
                                       @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("商铺名称") @RequestParam(value = "name", required = false) String name,
                                       HttpServletRequest request) throws Exception{

        Integer orgId = getOrgId(request);
        IPage<TaShopType> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaShopType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", orgId);
        queryWrapper.like(!StringUtils.isEmpty(name), "type_name", name);
        queryWrapper.orderByDesc("sort_no");

        IPage<TaShopType> result = iTaShopTypeService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taShopType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/shop-type",method= RequestMethod.POST)
    // @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taShopTypeAdd(@ApiParam("保存内容") @RequestBody TaShopType taShopType,
                                      HttpServletRequest request) throws Exception{

        Integer orgId = getOrgId(request);
        taShopType.setTypeId(CommonUtils.getUUIDS());
        taShopType.setOrgId(orgId);
        if (iTaShopTypeService.save(taShopType)){
            return ResponseBean.success(taShopType);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/shop-type/{id}", method= RequestMethod.DELETE)
    // @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taShopTypeDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaShopType taShopType = iTaShopTypeService.getById(id);
        taShopType.setStatus(CommConstant.STATUS_DELETE);
        if(iTaShopTypeService.updateById(taShopType)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taShopType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/shop-type/{id}",method= RequestMethod.PUT)
    // @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taShopTypeUpdate(@ApiParam("对象ID") @PathVariable String id,
                                         @ApiParam("更新内容") @RequestBody TaShopType taShopType) throws Exception{

        taShopType.setTypeId(id);
        if (iTaShopTypeService.updateById(taShopType)){
            return ResponseBean.success(iTaShopTypeService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/shop-type/{id}",method= RequestMethod.GET)
    // @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taShopTypeGet(@ApiParam("客户端") @PathVariable String client,
                                      @ApiParam("对象ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaShopTypeService.getById(id));
    }
}
